# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Local File Access module.

Define classes for local file access.
"""

import os
import stat
import locale
import shutil

from fileaccess import (FileObj, Access)


FILE_MODE = (stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP |
                stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)
DIR_MODE = stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO


class LocalFileObj(FileObj):
    """Class for local file."""

    def __init__(self, file_obj):
        """Constructor."""

        self.file_obj = file_obj

    def close(self):
        """Close file."""

        self.file_obj.close()

    def read(self, size=-1):
        """Return a "size" bytes data from current position in file.
        If the size argument is negative or omitted, then return all data.
        """

        return self.file_obj.read(size)

    def write(self, data):
        """Write a "data" data to current position in file."""

        self.file_obj.write(data)


class LocalAccess(Access):
    """class accessing to local file."""

    def _stat(self, target_path):
        """Return a stat information dictionary for "target_path"."""

        if isinstance(target_path, unicode):
            target_path = target_path.encode(self.encoding)
        stat_info = os.stat(target_path)
        return {
                stat.ST_MODE: stat_info.st_mode,
                stat.ST_MTIME: stat_info.st_mtime,
                stat.ST_CTIME: stat_info.st_ctime,
                stat.ST_SIZE: stat_info.st_size,
            }

    def __init__(self, base_dir=u"", encoding=None):
        """Constructor."""

        self.base_dir = base_dir
        self.encoding = encoding
        if not self.encoding:
            #self.encoding = locale.getpreferredencoding()
            self.encoding = "UTF-8"

    def make_directory(self, dir_path):
        """Create "dir_path" directory."""

        target_path = self.base_dir + dir_path
        if isinstance(target_path, unicode):
            target_path = target_path.encode(self.encoding)
        os.mkdir(target_path)
        os.chmod(target_path, DIR_MODE)

    def delete_directory(self, dir_path):
        """Delete "dir_path" directory."""

        target_path = self.base_dir + dir_path
        if isinstance(target_path, unicode):
            target_path = target_path.encode(self.encoding)
        shutil.rmtree(target_path)

    def stat_directory(self, dir_path):
        """Return a stat information dictionary for "file_path"
           which keys below:
            stat.ST_MODE, stat.ST_MTIME, stat.CTIME, stat.ST_SIZE
        """

        return self._stat(self.base_dir + dir_path)

    def rename_directory(self, dir_path, new_name):
        """Rename a name of "dir_path" to "new_name"."""

        _dir_path = self.base_dir + dir_path
        if isinstance(_dir_path, unicode):
            _dir_path = _dir_path.encode(self.encoding)
        os.rename(_dir_path, os.path.join(os.path.dirname(_dir_path),
                    new_name))

    def move_directory(self, dir_path, new_dir_path):
        """Move "dir_path" to "new_path"."""

        shutil.move(self.base_dir + dir_path, self.base_dir + new_dir_path)

    def get_contents(self, dir_path, with_stat=False):
        """Return a contents dictionary in "dir_path".
           A stat information must be included, if "with_stat" is True, 
        """

        _dir_path = self.base_dir + dir_path
        if isinstance(_dir_path, unicode):
            _dir_path = _dir_path.encode(self.encoding)
        contents = {}
        for name in os.listdir(_dir_path):
            _name = name
            if isinstance(name, str):
                name = name.decode(self.encoding)

            if with_stat:
                contents[name] = self._stat(os.path.join(_dir_path, _name))
            else:
                contents[name] = {}

        return contents

    def get_updated_contents(self, dir_path):
        """Return a updated contents dictionary in "dir_path"."""

        raise NotImplementedError

    def open(self, file_path, mode="r", length=-1):
        """Return a class instance of the "FileObj" or sub class
           corresponding to "file_path".
        """

        _file_path = self.base_dir + file_path
        if isinstance(_file_path, unicode):
            _file_path = _file_path.encode(self.encoding)
        file_obj = LocalFileObj(open(_file_path, mode))
        os.chmod(_file_path, FILE_MODE)
        return file_obj

    def delete_file(self, file_path):
        """Delete "file_path" file."""

        target_path = self.base_dir + file_path
        if isinstance(target_path, unicode):
            target_path = target_path.encode(self.encoding)
        os.unlink(target_path)

    def stat_file(self, file_path):
        """Return a stat information dictionary for "file_path"
           which keys below:
            stat.ST_MODE, stat.ST_MTIME, stat.CTIME, stat.ST_SIZE
        """

        return self._stat(self.base_dir + file_path)

    def rename_file(self, file_path, new_name):
        """Rename a name of "file_path" to "new_name"."""

        _file_path = self.base_dir + file_path
        if isinstance(_file_path, unicode):
            _file_path = _file_path.encode(self.encoding)
        os.rename(_file_path, os.path.join(os.path.dirname(_file_path),
                    new_name))

    def move_file(self, file_path, new_dir_path):
        """Move "file_path" to "new_path"."""

        shutil.move(self.base_dir + file_path, self.base_dir + new_dir_path)
